#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <gccore.h>
#include <ogc/lwp_watchdog.h>

#include <fat.h>
#include <wiisprite.h>
#include <wiiuse/wpad.h>

#include "HomeMenu.h"

#define TXT_X 334
#define TXT_Y 354
#define ICON_X 560
#define ICON_Y 425
#define RULER_X 302
#define RULER_Y 123
#define RULER_SIZE 6
#define BALL_X 320
#define BALL_Y 218
#define BALL_SENSE_X 8
#define BALL_SENSE_Y 5
#define DISPLAY_X 262
#define DISPLAY_Y 110
#define DIGITS_X 12
#define DIGITS_Y 14
#define DIGIT_W 30
#define LBS_MULTIPLIER 2.20462278 //taken from Wikipedia.org

//#include "GRRLIB/GRRLIB.h"

#define DATAPATH "sd:/apps/balancetools/images/"
#define CONFPATH "sd:/apps/balancetools/baltools.cfg"

enum screen_names {WIN_MAIN, WIN_WEIGHT, WIN_COG};

static void *xfb = NULL;
static GXRModeObj *rmode = NULL;

//############ LIBWIISPRITE #############
using namespace wsp;
//########## END LIBWIISPRITE ###########

//options
bool digital=false, lbs=false;
s8  offset=0,target=0;

//############ SAVE STUFF #############
//
// stolen from Mahjongg Wii 0.8 Source
// Code     : Justin Wood
//
bool loadConfig()
{//*
	if(!fatInitDefault()) return false;
	FILE *fp;

	if((fp = fopen(CONFPATH,"rb"))==NULL) return false;

	lbs=getc(fp)==1?true:false;
	digital=getc(fp)==1?true:false;
	offset=getc(fp);
	target=getc(fp);

	fclose(fp);
	return true;//*/
}

void saveConfig()
{//*
	if(!fatInitDefault()) return;
 
	FILE *fp;

	if((fp = fopen(CONFPATH,"wb"))==NULL) return;
	
	putc(lbs?1:0, fp);
	putc(digital?1:0, fp);
	putc(offset, fp);
	putc(target, fp);

	fclose(fp);//*/
}
//############ SAVE STUFF #############

void exit0(u8 num)
{	
	switch (num)
	{
	case 0: printf("Unknown error"); break;
	case 1: printf("error loading image");break;
	case 2: printf("variable error");break;
	case 4: printf("Exit Pressed");break;
	}
	saveConfig();
	exit(0);
}

int main(int argc, char **argv)
{
	bool done=false, rumbeling = false, bbcheck = false;
	//############ LIBWIISPRITE #############
	GameWindow gwd; 
	Sprite	cursor,
			txtsystem, //text boxes
			ruler, //the ruler sprite
			button1, button2, button3, buttonback, //buttons
			sprback1,sprback2,sprback3, sprtarget,//backgrounds
			sprball, //the red ball in the center of gravity
			spricon, //balance board icon on the main screen
			sprdisplay,sprdig[4]; //the digital display
	Image	imgcursor, imgball, imgrulerkg,imgrulerlb, imgicon, imgkg,imglbs,
			imgbtn1, imgbtn2, imgbtn3,  imgbtnback,
			imgback1, imgback2, imgback3,
			imgdigits[10],imgtarget[9],imgdigitse,imgdisplay;
	LayerManager manager(6),cog(5),weight(10);	
	//########## END LIBWIISPRITE ###########
	
	u64 time2rumble = 0;
	s8 selected = 0, screen=WIN_MAIN, current = -1;
	u8 i, digits[4];
	int bbport=0;
	
	expansion_t exp;
	float x=0,y=0,weightx;

	fatInitDefault();

	gwd.InitVideo();

	rmode = VIDEO_GetPreferredMode(NULL);
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	printf("\x1b[2;0H"); //get the cursor to (2,0)

	//if( CONF_GetAspectRatio() )
	//{
	//	rmode->viWidth = 678;
	//	rmode->viXOrigin = (VI_MAX_WIDTH_PAL - 678)/2;
	//}
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(false); //make console stuff visible
	VIDEO_Flush();
	VIDEO_WaitVSync();	
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

	printf("Initialize Video...\n");
	GX_InvalidateTexAll();
	
	gwd.InitVideo();
	printf("Loading Config File... ");
		if (!loadConfig())	printf("[Config file not found, will be created on exit!]");
	printf("\nInitialize HOMEMenu...\n");
	HomeMenu_Init(640,480,gwd.GetFrameBuffer(0),gwd.GetFrameBuffer(1),gwd.GetFrameBufferIndex());
	HomeMenu_SetGFX(HM_GFX_LIBWIISPRITE);
	HomeMenu_SetSND(HM_SND_ASND);
	
	printf("Initialize images...\n");
	printf("\t00/34-cursor.png\x1b[6;0H");
	if(imgcursor.LoadImage(DATAPATH "cursor.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t01/34-floor.png\x1b[6;0H");
	if(imgback1.LoadImage(DATAPATH "floor.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t02/34-btn-weight.png\x1b[6;0H");
	if(imgbtn1.LoadImage(DATAPATH "btn-weight.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t03/34-btn-cog.png\x1b[6;0H");
	if(imgbtn2.LoadImage(DATAPATH "btn-cog.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t04/34-btn-exit.png\x1b[6;0H");
	if(imgbtn3.LoadImage(DATAPATH "btn-exit.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t05/34-btn-back.png\x1b[6;0H");
	if(imgbtnback.LoadImage(DATAPATH "btn-back.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t06/34-fit-weight.png\x1b[6;0H");
	if(imgback2.LoadImage(DATAPATH "fit-weight.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t07/34-fit-cog.png\x1b[6;0H");
	if(imgback3.LoadImage(DATAPATH "fit-cog.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t08/34-ball.png\x1b[6;0H");
	if(imgball.LoadImage(DATAPATH "ball.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t09/34-lbruler.png\x1b[6;0H");
	if(imgrulerlb.LoadImage(DATAPATH "lbruler.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t10/34-kgruler.png\x1b[6;0H");
	if(imgrulerkg.LoadImage(DATAPATH "kgruler.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t11/34-ico-bb.png\x1b[6;0H");
	if(imgicon.LoadImage(DATAPATH "ico-bb.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t12/34-txt-kg.png\x1b[6;0H");
	if(imgkg.LoadImage(DATAPATH "txt-kg.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t13/34-txt-lbs.png\x1b[6;0H");
	if(imglbs.LoadImage(DATAPATH "txt-lbs.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	//digital display images
	printf("\t14/34-dig-display.png\x1b[6;0H");
	if(imgdisplay.LoadImage(DATAPATH "dig-display.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t15/34-dig-0.png\x1b[6;0H");
	if(imgdigits[0].LoadImage(DATAPATH "dig-0.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t16/34-dig-1.png\x1b[6;0H");
	if(imgdigits[1].LoadImage(DATAPATH "dig-1.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t17/34-dig-2.png\x1b[6;0H");
	if(imgdigits[2].LoadImage(DATAPATH "dig-2.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t18/34-dig-3.png\x1b[6;0H");
	if(imgdigits[3].LoadImage(DATAPATH "dig-3.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t19/34-dig-4.png\x1b[6;0H");
	if(imgdigits[4].LoadImage(DATAPATH "dig-4.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t20/34-dig-5.png\x1b[6;0H");
	if(imgdigits[5].LoadImage(DATAPATH "dig-5.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t21/34-dig-6.png\x1b[6;0H");
	if(imgdigits[6].LoadImage(DATAPATH "dig-6.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t22/34-dig-7.png\x1b[6;0H");
	if(imgdigits[7].LoadImage(DATAPATH "dig-7.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t23/34-dig-8.png\x1b[6;0H");
	if(imgdigits[8].LoadImage(DATAPATH "dig-8.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t24/34-dig-9.png\x1b[6;0H");
	if(imgdigits[9].LoadImage(DATAPATH "dig-9.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t25/34-dig-empty.png\x1b[6;0H");
	if(imgdigitse.LoadImage(DATAPATH "dig-empty.png") != IMG_LOAD_ERROR_NONE) exit0(1); 
	printf("\t26/34-target0.png\x1b[6;0H");
	if(imgtarget[0].LoadImage(DATAPATH "target0.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t27/34-target1.png\x1b[6;0H");
	if(imgtarget[1].LoadImage(DATAPATH "target1.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t28/34-target2.png\x1b[6;0H");
	if(imgtarget[2].LoadImage(DATAPATH "target2.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t29/34-target3.png\x1b[6;0H");
	if(imgtarget[3].LoadImage(DATAPATH "target3.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t30/34-target4.png\x1b[6;0H");
	if(imgtarget[4].LoadImage(DATAPATH "target4.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t31/34-target5.png\x1b[6;0H");
	if(imgtarget[5].LoadImage(DATAPATH "target5.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t32/34-target6.png\x1b[6;0H");
	if(imgtarget[6].LoadImage(DATAPATH "target6.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t33/34-target7.png\x1b[6;0H");
	if(imgtarget[7].LoadImage(DATAPATH "target7.png") != IMG_LOAD_ERROR_NONE) exit0(1);
	printf("\t34/34-target8.png\x1b[6;0H");
	if(imgtarget[8].LoadImage(DATAPATH "target8.png") != IMG_LOAD_ERROR_NONE) exit0(1);

	printf("Initialize Sprites...\n");
	cursor.SetImage(&imgcursor);cursor.SetPosition(320,240);
	sprback1.SetImage(&imgback1);sprback1.SetPosition(0,-5);sprback1.SetVisible(true);
	sprback2.SetImage(&imgback2);sprback2.SetPosition(0,-5);sprback2.SetVisible(true);
	sprback3.SetImage(&imgback3);sprback3.SetPosition(0,-5);sprback3.SetVisible(true);
	sprtarget.SetImage(&imgtarget[0]);sprtarget.SetPosition(0,-5);sprtarget.SetVisible(true);
	button1.SetImage(&imgbtn1);button1.SetPosition(60,200);button2.SetImage(&imgbtn2);	
	button2.SetPosition(60,280);button3.SetImage(&imgbtn3);button3.SetPosition(60,360);
	buttonback.SetImage(&imgbtnback);buttonback.SetPosition(5,410);buttonback.SetVisible(true);
	sprball.SetImage(&imgball);	sprball.SetPosition(BALL_X,BALL_Y);	sprball.SetVisible(true);
	//initialize ruler images
	if (!lbs) {	ruler.SetImage(&imgrulerkg); txtsystem.SetImage(&imgkg);}
	else {	ruler.SetImage(&imgrulerlb); txtsystem.SetImage(&imglbs); }
		ruler.SetPosition(RULER_X,RULER_Y);	ruler.SetVisible(true);
		txtsystem.SetPosition(TXT_X,TXT_Y);	txtsystem.SetVisible(true);
	spricon.SetImage(&imgicon);	spricon.SetPosition(ICON_X,ICON_Y);	spricon.SetVisible(true);
	sprdisplay.SetImage(&imgdisplay);sprdisplay.SetPosition(DISPLAY_X,DISPLAY_Y);sprdisplay.SetVisible(digital);
	sprdig[0].SetImage(&imgdigitse);sprdig[0].SetPosition(DISPLAY_X+DIGITS_X,DISPLAY_Y+DIGITS_Y);	sprdig[0].SetVisible(digital);
	sprdig[1].SetImage(&imgdigitse);sprdig[1].SetPosition(DISPLAY_X+DIGIT_W+DIGITS_X,DISPLAY_Y+DIGITS_Y);sprdig[1].SetVisible(digital);
	sprdig[2].SetImage(&imgdigitse);sprdig[2].SetPosition(DISPLAY_X+DIGIT_W*2+DIGITS_X,DISPLAY_Y+DIGITS_Y);sprdig[2].SetVisible(digital);
	sprdig[3].SetImage(&imgdigitse);sprdig[3].SetPosition(DISPLAY_X+DIGIT_W*3+DIGITS_X,DISPLAY_Y+DIGITS_Y);sprdig[3].SetVisible(digital);
		
	manager.Append(&cursor);
	manager.Append(&spricon);
	manager.Append(&button1);
	manager.Append(&button2);
	manager.Append(&button3);
	manager.Append(&sprback1);	
	
	cog.Append(&cursor);
	cog.Append(&sprball);
	cog.Append(&buttonback);
	cog.Append(&sprtarget);
	cog.Append(&sprback3);
	
	weight.Append(&cursor);
	weight.Append(&sprdig[3]);
	weight.Append(&sprdig[2]);
	weight.Append(&sprdig[1]);
	weight.Append(&sprdig[0]);
	weight.Append(&sprdisplay); //base
	weight.Append(&buttonback);
	weight.Append(&txtsystem);
	weight.Append(&sprback2);
	weight.Append(&ruler);
	
	printf("initialize Wiimote...\n");
	// Initialise Wiimote
	WPAD_Init();
	WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);
	WPAD_SetVRes(0,rmode->fbWidth,rmode->xfbHeight);
	
	while(!done)
	{
		WPAD_ScanPads();
		for (i=0;i<WPAD_MAX_WIIMOTES;i++) //always check that's its there
		{
			u32 devtype;
			WPAD_Probe(i,&devtype);
			if (devtype==WPAD_EXP_WIIBOARD)
			{
				spricon.SetTransparency(255);
				bbport=i;
				bbcheck=true;
				break;
			}
		}
		if (!bbcheck)
		{ spricon.SetTransparency(92); x=0; y=0; }
		else
		{	WPAD_Expansion(bbport, &exp);
			x=exp.wb.x;
			y=exp.wb.y;
		}
		
		// INFRA-RED CODE //////////////////////////////////
		ir_t ir;
		WPAD_IR(WPAD_CHAN_0, &ir);
		cursor.SetPosition(ir.sx-WSP_POINTER_CORRECTION_X, ir.sy-WSP_POINTER_CORRECTION_Y);
		cursor.SetRotation(ir.angle/2);

		// SCREEN ROUTINES ////////////////////////////////

		switch(screen)
		{	case WIN_MAIN:
			
			if	 (cursor.CollidesWith(&button1))
			{ 	button1.SetTransparency(255);
				button2.SetTransparency(127);
				button3.SetTransparency(127);
				selected = 1;  
			}
			else if(cursor.CollidesWith(&button2))
			{ 	button2.SetTransparency(255);
				button1.SetTransparency(127);
				button3.SetTransparency(127);
				selected = 2; 
			}
			else if(cursor.CollidesWith(&button3))
			{	button3.SetTransparency(255);
				button1.SetTransparency(127);
				button2.SetTransparency(127);
				selected = 3; 
			}
			else
			{ 	button1.SetTransparency(127);
				button2.SetTransparency(127);
				button3.SetTransparency(127);
				selected = 0; current = -1; 
			}
			manager.Draw(0,0);
			break;
			
			case WIN_COG:
			
			sprball.SetPosition(BALL_X+x*BALL_SENSE_X,BALL_Y+y*BALL_SENSE_Y);
			if	 (cursor.CollidesWith(&buttonback))
			{ 	buttonback.SetTransparency(255);
				selected=4;
			}
			else 
			{ 	buttonback.SetTransparency(127);
				selected=0; current = -1; 
			}			
			cog.Draw(0,0);
			break;
			
			case WIN_WEIGHT:
			
			weightx=exp.wb.tl+exp.wb.tr+exp.wb.bl+exp.wb.br;
			if (weightx<0.1) weightx=0;
			if (digital)
			{	//DIGITAL WEIGHT DISPLAY ###DIG###
				
				sprdig[0].SetImage(&imgdigitse);
				sprdig[1].SetImage(&imgdigitse);
				sprdig[2].SetImage(&imgdigitse);
				sprdig[3].SetImage(&imgdigitse);
				if (lbs) weightx=weightx*LBS_MULTIPLIER;
				digits[0]=floor(weightx/100);
				digits[1]=floor((weightx-digits[0]*100)/10);
				digits[2]=floor(weightx-digits[0]*100-digits[1]*10);
				digits[3]=floor((weightx-floor(weightx))*10);
				if (weightx>99.9)
					for (i=0;i<4;i++) sprdig[i].SetImage(&imgdigits[digits[i]]);
				else if (weightx>9.9)
					for (i=1;i<4;i++) sprdig[i].SetImage(&imgdigits[digits[i]]);
				else
					for (i=2;i<4;i++) sprdig[i].SetImage(&imgdigits[digits[i]]);
			} 
			else //ANALOG WEIGHT DISPLAY ###ANA###
				ruler.SetPosition(offset+RULER_X-RULER_SIZE*weightx,RULER_Y);
			
			if	 (cursor.CollidesWith(&buttonback))
			{ 	buttonback.SetTransparency(255);
				selected=4;
			}
			else 
			{ 	buttonback.SetTransparency(127);
				selected=0; current = -1; 
			}
			weight.Draw(0,0);
			break;
		}
		// RUMBLE CODE //////////////////////////////////
		if(selected>0 && selected!=current)
		{
			if(!rumbeling){
				time2rumble = ticks_to_millisecs(gettime());
				WPAD_Rumble(0,1);
				rumbeling = true;
			}
			current = selected;
		}
			//stop rumble after 50ms
		if(ticks_to_millisecs(gettime())>time2rumble+50 && rumbeling)
		{
			WPAD_Rumble(0,0);
		}
			//let it rumble again after 250ms
		if(ticks_to_millisecs(gettime())>time2rumble+250 && rumbeling)
		{	
			rumbeling = false;
		}
		
		// BUTTON PRESS CODE //////////////////////////////////
		
		if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_A) && selected>0)
		{  
			switch(selected)
			{
				// there is also SYS_ResetSystem() in system.c
				case 1: screen=WIN_WEIGHT;break;
				case 2:  screen=WIN_COG; break;
				case 3: done=true; break;
				case 4:  screen=WIN_MAIN;break;
				default: break;
			}
			switch (screen)
			{	case WIN_MAIN: manager.Draw(0,0); break;
				case WIN_COG: cog.Draw(0,0); break;
				case WIN_WEIGHT: weight.Draw(0,0); break;
			}
			
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_MINUS) && (screen==WIN_WEIGHT))
		{	if (offset!=-128) offset--;
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_PLUS) && (screen==WIN_WEIGHT))
		{	if (offset!=127) offset++;
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_MINUS) && (screen==WIN_COG) && (target>0))
		{	target--;
			sprtarget.SetImage(&imgtarget[target]);
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_PLUS) && (screen==WIN_COG) && (target<8))
		{	target++;
			sprtarget.SetImage(&imgtarget[target]);
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_1) && (screen==WIN_WEIGHT))
		{	digital=!digital;
			sprdisplay.SetVisible(digital);
			sprdig[0].SetVisible(digital);
			sprdig[1].SetVisible(digital);
			sprdig[2].SetVisible(digital);
			sprdig[3].SetVisible(digital);
		}
		else if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_A) && (selected==0))
		{	if (lbs)
			{	lbs=false;
				ruler.SetImage(&imgrulerkg);
				txtsystem.SetImage(&imgkg);
			}
			else
			{	lbs=true;
				ruler.SetImage(&imgrulerlb);
				txtsystem.SetImage(&imglbs);
			}
		}
		else if (WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_HOME)
		{
			HomeMenu_Show();
		}
		
		//draw the screen
		gwd.Flush();
	}
	WPAD_Disconnect(WPAD_CHAN_0);
	//WPAD_Shutdown();
	exit0(4);
}

